unit PingThrd1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Ping, StdCtrls, ExtCtrls;

type
  TPingThread = class(TThread)
  private
    FPing: TPing;
    procedure FPingDnsLookupDone(Sender: TObject; Error: Word);
    procedure FPingEchoReply(Sender, Icmp: TObject; Error: Integer);
  public
    Addr: string;
    PingResult: string;
    procedure Execute; override;
  end;

type
  TForm1 = class(TForm)
    Memo1: TMemo;
    Panel1: TPanel;
    Edit1: TEdit;
    PingBtn: TButton;
    procedure PingBtnClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  private
    PingThreads: array of Cardinal;
    procedure PingThreadTerminate(Sender: TObject);
  public
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

//------------------------------------------------------------------------------
//---- TPingThread -------------------------------------------------------------
//------------------------------------------------------------------------------
procedure TPingThread.Execute;
var
   Msg: TagMsg;
begin
   FPing := TPing.Create(nil);      // create in thread context
   FPing.OnDnsLookupDone := FPingDnsLookupDone;
   FPing.DnsLookup(Addr);

   while not Terminated do
   begin
      //if PeekMessage(Msg, 0, 0, 0, PM_REMOVE) then begin
      GetMessage(Msg, 0, 0, 0);     // create a message queue
      if Msg.message = WM_QUIT then
         Terminate;
      TranslateMessage(Msg);
      DispatchMessage(Msg);
   end;

   FPing.Destroy;
end;

//------------------------------------------------------------------------------
procedure TPingThread.FPingDnsLookupDone(Sender: TObject; Error: Word);
begin
   if Error <> 0 then
   begin
      PingResult := 'Unknown host ''' + Addr + '''';
      Terminate;
      Exit;
   end;

   with Sender as TPing do
   begin
      Address     := DnsResult;
      OnEchoReply := FPingEchoReply;
      Ping;
   end;
end;

//------------------------------------------------------------------------------
procedure TPingThread.FPingEchoReply(Sender, Icmp: TObject; Error: Integer);
begin
   with Sender as TPing do
      if Error = 0 then
         PingResult := 'Cannot ping host (' + HostIP + ') : ' + ErrorString
      else
         PingResult := 'Received ' + IntToStr(Reply.DataSize) + ' bytes from ' + HostIP + ' in ' + IntToStr(Reply.RTT) + ' ms';
   Terminate;
end;

//------------------------------------------------------------------------------
//--- TForm1 -------------------------------------------------------------------
//------------------------------------------------------------------------------
procedure TForm1.PingThreadTerminate(Sender: TObject);
begin
   if not Application.Terminated then
      with Sender as TPingThread do
         Memo1.Lines.Add(PingResult);
end;

//------------------------------------------------------------------------------
procedure TForm1.PingBtnClick(Sender: TObject);
begin
   with TPingThread.Create(True) do
   begin
      SetLength(PingThreads, Length(PingThreads) + 1);
      PingThreads[Length(PingThreads) - 1] := ThreadId;
      FreeOnTerminate := True;
      OnTerminate := PingThreadTerminate;
      Addr := Edit1.Text;
      Resume;
   end;
end;

//------------------------------------------------------------------------------
procedure TForm1.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
var
   n: integer;
begin
   for n := 0 to Length(PingThreads) - 1 do
      PostThreadMessage(PingThreads[n], WM_QUIT, 0, 0);  // terminate all threads
   PingThreads := nil;                                   // free the thread id array
end;

end.
